% Kernel learning via quadratically constrained quadratical programming (QCQP) in binary case
% Coded by Shuiwang Ji (shuiwang.j@asu.edu, Arizona State University)
% Modified by Jianhui Chen (jianhui.chen@asu.edu, Arizona State University)

%                Input Parameters
% "G" : the cell containing centered kernel matrix (trace value = 1) 
% "lambda" : regularization parameter
% "a" : the indicator vector

%               Output parameters
% "theta" : the coefficients of kernel matrices
% "time" : computation time in seconds

function [theta, time, beta, r_value, res] = QCQP_wo_tau_binary(G, a, lambda)

t0 = cputime;

p = length(G);

n = length(a);

clear prob;
%   specify c
prob.c = [a; -1 / (4*lambda)];


%   specify quadratic terms in sparse format in the objective
% prob.qosubi = 1:n+1;
% prob.qosubj = 1:n+1;
% prob.qoval = [- 1 / 2 * ones(n, 1); 0];


prob.qosubi = [1:n]';
prob.qosubj = [1:n]';
prob.qoval = - 1 / 2 * ones(n,1);



%   specify quadratic terms in sparse format in the constraints
prob.qcsubk = [];
prob.qcsubi = [];
prob.qcsubj = [];
prob.qcval = [];
for i = 1:p
    [row_num, col_num, entry_val] = find(tril(G{i}));
    prob.qcsubk = [prob.qcsubk; i*ones(length(entry_val), 1)];
    prob.qcsubi = [prob.qcsubi; row_num];
    prob.qcsubj = [prob.qcsubj; col_num];
    prob.qcval = [prob.qcval; (2.*entry_val)];
end

%   specify linear terms in sparse format in the constraints
prob.a = [sparse(p, n) -1*ones(p,1)];

% lower bounds on constraints
prob.blc = [-inf*ones(p, 1)];

% upper bounds on constraints
prob.buc = zeros(p, 1);

% call Mosek optimization engine
%[r_value, res] = mosekopt('maximize info',prob);
[r_value, res] = mosekopt('maximize info',prob);
beta = res.sol.itr.xx(1:n, 1);

%mu_opt = res.sol.itr.y(1:m_numb,1)./r; 
theta = res.sol.itr.y * (4*lambda);

%time = res.info.MSK_DINF_INTPNT_CPUTIME;

time = cputime - t0;

% t = res.sol.itr.xx(end);
% 
% for i = 1:10
%     tt(i) = beta' * G{i} * beta;
% end

















